from vcCommand import *
import openpyxl
import os

cmd = getCommand()
app = getApplication()
sim = getSimulation()

cmd.createProperty(VC_URI, "SheetPath")
button = cmd.createProperty(VC_BUTTON, "Load Link Data")
writeButton = cmd.createProperty(VC_BUTTON, "Write Link Data")
text = cmd.createProperty(VC_STRING, "NOTE")
text.Value = "Write button will create Excel file."
text.WritableWhenConnected = False
text.WritableWhenDisconnected = False
text.WritableWhenSimulating = False

def get_file_path():
  sheet_path = cmd.getProperty("SheetPath").Value
  if sheet_path.startswith('file:///'):
    return sheet_path[8:]
  elif sheet_path == "":
    return None
  else:
    return sheet_path

def ReadLayoutLinks():
  sourceDestList = []
  LinkedList = sim.ProcessController.TransportSystem.Links
  for x in LinkedList:
    if "Transport Node" in x.Source.Component.Name or "Transport Node" in x.Destination.Component.Name:
      if "Transport Node" in x.Source.Component.Name and "Transport Node" in x.Destination.Component.Name:
        sourceDestCombo = [x.Source.Component.Name, x.Destination.Component.Name, x.SupportedGroup.Name]
      elif "Transport Node" in x.Source.Component.Name:
        sourceDestCombo = [x.Source.Component.Name, x.Destination.ProcessExecutor.Processes[0].Name,x.SupportedGroup.Name]
      else:
        sourceDestCombo = [x.Source.ProcessExecutor.Processes[0].Name,x.Destination.Component.Name,x.SupportedGroup.Name]
    else:
      sourceDestCombo = [x.Source.ProcessExecutor.Processes[0].Name, x.Destination.ProcessExecutor.Processes[0].Name,x.SupportedGroup.Name]
    if sourceDestCombo not in sourceDestList:
      sourceDestList.append(sourceDestCombo)
  return sourceDestList,LinkedList

def ReadExcelLinks(file_path):
  wb = openpyxl.load_workbook(file_path)
  ws = wb.worksheets[0]
  target = 'A2'
  sourceDestList = []
  LinkedList = sim.ProcessController.TransportSystem.Links
  while ws[target].value != None:
    i = target[1:]
    source = ws[target].value
    dest = ws['B'+ target[1:]].value
    flowgroup = ws['G'+target[1:]].value
    row = target[1:]
    sourceDestList.append([source,dest,row,flowgroup])
    target = 'A'+str(int(i)+1)
  wb.close()  
  return sourceDestList,LinkedList


def WriteLayoutLinks(sourceDestList,LinkedList,file_path):
  wb = openpyxl.load_workbook(file_path)
  ws = wb.worksheets[0]
  for x in LinkedList:
    for y in sourceDestList: 
      comboFound = False
      if "Transport Node" in x.Source.Component.Name or "Transport Node" in x.Destination.Component.Name:
        if "Transport Node" in x.Source.Component.Name and "Transport Node" in x.Destination.Component.Name:
          if x.Source.Component.Name == y[0] and x.Destination.Component.Name == y[1] and x.SupportedGroup.Name == y[3]:
            comboFound = True
        elif "Transport Node" in x.Source.Component.Name:
          if x.Source.Component.Name == y[0] and x.Destination.ProcessExecutor.Processes[0].Name == y[1] and x.SupportedGroup.Name == y[3]:
            comboFound = True
        elif "Transport Node" in x.Destination.Component.Name:
          if x.Source.ProcessExecutor.Processes[0].Name == y[0] and x.Destination.Component.Name == y[1] and x.SupportedGroup.Name == y[3]:
            comboFound = True
      elif x.Source.ProcessExecutor.Processes[0].Name == y[0] and x.Destination.ProcessExecutor.Processes[0].Name == y[1] and x.SupportedGroup.Name == y[3]:
        comboFound = True
      if comboFound:
        if ws['C'+y[2]].value == "Interpolating":
          transpTime = x.getProperty("TransportTime")
          transpTime.Value = float(ws['F'+y[2]].value)
        else:
          if "Single Rail" in x.Implementer.Component.Name or "Dual Rail" in x.Implementer.Component.Name:
            pickTime = x.getProperty("GraspTime")
            placeTime = x.getProperty("ReleaseTime")
          else:
            pickTime = x.getProperty("PickTime")
            placeTime = x.getProperty("PlaceTime")
          pickTime.Value = float(ws['D'+y[2]].value)
          placeTime.Value = float(ws['E'+y[2]].value)
  wb.save(file_path)
  return "Wrote Tranport Links to layout."

def WriteExcelLinks(sourceDestList,LinkedList):
  #linked excel file? T: go through and write excel links to the linked file. F: make new excel file, link it to the comp property, and write links.
  file_path = get_file_path()
  if file_path == None:
    wb = openpyxl.Workbook()
  else:
    wb = openpyxl.load_workbook(file_path)
  ws = wb.worksheets[0]
  ws['A1'] = "Source"
  ws['B1'] = "Destination"
  ws['C1'] = "Implementer"
  ws['D1'] = "PickTime"
  ws['E1'] = "PlaceTime"
  ws['F1'] = "TransportTime"
  ws['G1'] = "Flow Group"
  target = 'A2'
  if LinkedList == []:
    return("No Transport Links in layout.")
  else:
    for x in LinkedList:
      for y in sourceDestList:
        comboFound = False
        if "Transport Node" in x.Source.Component.Name or "Transport Node" in x.Destination.Component.Name:
          if "Transport Node" in x.Source.Component.Name and "Transport Node" in x.Destination.Component.Name:
            if x.Source.Component.Name == y[0] and x.Destination.Component.Name == y[1]:
              comboFound = True
          elif "Transport Node" in x.Source.Component.Name:
            if x.Source.Component.Name == y[0] and x.Destination.ProcessExecutor.Processes[0].Name == y[1]:
              comboFound = True
          elif "Transport Node" in x.Destination.Component.Name:
            if x.Source.ProcessExecutor.Processes[0].Name == y[0] and x.Destination.Component.Name == y[1]:
              comboFound = True
        elif x.Source.ProcessExecutor.Processes[0].Name == y[0] and x.Destination.ProcessExecutor.Processes[0].Name == y[1]:
          comboFound = True
        #need case for when not transport node
        if comboFound:
          if(str(type(x.Implementer)) == '<type \'vcPythonTransportController\'>'):
            #print(x.Implementer.Component.Name)
            if "Single Rail" in x.Implementer.Component.Name or "Dual Rail" in x.Implementer.Component.Name:
              pickTime = x.getProperty("GraspTime")
              placeTime = x.getProperty("ReleaseTime")
            else:
              pickTime = x.getProperty("PickTime")
              placeTime = x.getProperty("PlaceTime")
            ws['A'+str(target[1:])] = y[0] #source
            ws['B'+str(target[1:])] = y[1] #destination
            ws['C'+str(target[1:])] = "TC"
            ws['D'+str(target[1:])] = pickTime.Value
            ws['E'+str(target[1:])] = placeTime.Value
            ws['F'+str(target[1:])] = ""
            ws['G'+str(target[1:])] = y[2] #flow group 
            rowIncrement = int(target[1:])+1
            target = target[0]+str(rowIncrement)
          elif(str(type(x.Implementer)) == '<type \'vcInterpolatingTransportController\'>'):
            #print('interp')
            ws['A'+str(target[1:])] = y[0] #source
            ws['B'+str(target[1:])] = y[1] #destination
            ws['C'+str(target[1:])] = "Interpolating"
            ws['D'+str(target[1:])] = ""
            ws['E'+str(target[1:])] = ""
            ws['F'+str(target[1:])] = x.Properties[4].Value
            ws['G'+str(target[1:])] = y[2] #flow group
            rowIncrement = int(target[1:])+1
            target = target[0]+str(rowIncrement)            
          elif(str(type(x.Implementer)) == '<type \'vcTransportController\'>'):
            pass
            #don't need to add this to sheet but may as well hold onto the way to get a handle on it in case of future development on this.
          sourceDestList.remove(y)
    if cmd.getProperty("SheetPath").Value[8:] != "":
      if cmd.getProperty("SheetPath").Value[:8] == 'file:///':
        wb.save(cmd.getProperty("SheetPath").Value[8:])
      else:
        wb.save(cmd.getProperty("SheetPath").Value)
    else:
      desktop_path = os.path.join(os.path.expanduser('~'), 'Downloads')
      file_name = 'LinkData.xlsx'
      file_path = os.path.join(desktop_path, file_name)
      file_path.replace('\\\\','\\')
      wb.save(file_path)
      cmd.SheetPath = file_path
    return("Wrote Transport Links to file.")

def loadLinkData(prop):
  file_path = get_file_path()
  if file_path == None:
    print("No file linked.")
  else: 
    sourceDestList,LinkedList = ReadExcelLinks(file_path)
    msg = WriteLayoutLinks(sourceDestList,LinkedList,file_path)
    print(msg)  

def writeLinkData(prop):
  sourceDestList,LinkedList = ReadLayoutLinks()
  response = WriteExcelLinks(sourceDestList,LinkedList)
  print(response)

def OnStart():
  executeInActionPanel()
writeButton.OnChanged = writeLinkData
button.OnChanged = loadLinkData